#include <kpacman.h>
#include <kpacman.moc>

#include <qkeycode.h>
#include <qcolor.h>
#include <qstring.h>
#include <qpopmenu.h>
#include <qmsgbox.h>

#include <kcolordlg.h>

KPacman::KPacman(QWidget *, const char *name)
	: KTopLevelWidget()
{
    setCaption("Pacman");
    setIcon("Pacman");

    conf = kapp->getConfig();
    if(conf == NULL) {
	printf(i18n("KConfig error ??\n"));
	kapp->quit();
    }

    menu();
 
    view = new KPacmanWidget(this, name, conf, conf->readNumEntry("Scheme", -1));
    referee = view->referee;
    referee->setFocus();

    connect(referee, SIGNAL(setPoints(int)), view->score, SLOT(set(int)));
    connect(referee, SIGNAL(setLifes(int)), view->status, SLOT(setLifes(int)));
    connect(referee, SIGNAL(setLevel(int)), view->status, SLOT(setLevel(int)));

    connect(referee, SIGNAL(togglePaused()), this, SLOT(togglePaused()));

    menuBar->show();
    setMenu(menuBar);
    view->show();
    setView(view);
}

KPacman::~KPacman()
{
    delete menuBar;
}

void KPacman::menu()
{

    game = new QPopupMenu();
    CHECK_PTR( game );
    game->insertItem(i18n("&New"), this, SLOT(newKPacman()),Key_F2);
    pauseID = game->insertItem(i18n("&Pause"), this , SLOT(pauseKPacman()), Key_F3);
    game->insertSeparator();
    game->insertItem( i18n("&Quit"),  this, SLOT(quitKPacman()), CTRL+Key_Q );
    game->setCheckable( TRUE );

    options = new QPopupMenu();
    CHECK_PTR( options );

    if (lookupSchemes() > 0) {
	options->insertItem(i18n("&Select graphic scheme"),  schemes);
	options->insertSeparator();
    }

    options->insertItem(i18n("Change &keys..."), this, SLOT(confKeys()));

    options->setCheckable( TRUE );

    QString msg = i18n("@PACKAGE@ - @VERSION@\n\n"
		       "Joerg Thoennissen (joe@joshua.dut.dirnet.com)\n\n"
                       "A pacman game for the KDE Desktop\n\n"
		       "The program based on the source of ksnake\n"
		       "by Michel Filippi (mfilippi@sade.rhein-main.de).\n"
		       "The design was strongly influenced by the pacman\n"
		       "(c) 1980 MIDWAY MFG.CO.\n\n"
		       "I like to thank my girlfriend Elke Krueers for\n"
		       "the last 10 years of her friendship.\n");
    msg.replace(QRegExp("@PACKAGE@"), PACKAGE);
    msg.replace(QRegExp("@VERSION@"), VERSION);
    QPopupMenu *help = kapp->getHelpMenu(FALSE, msg);	

    menuBar = new KMenuBar( this );
    CHECK_PTR( menuBar );
    menuBar->insertItem(i18n("&Game"), game );
    menuBar->insertItem(i18n("&Options"), options );
    menuBar->insertSeparator();
    menuBar->insertItem(i18n("&Help"), help);
}


int KPacman::lookupSchemes()
{
    int SchemeCount = conf->readNumEntry("SchemeCount");
    int Scheme = conf->readNumEntry("Scheme", -1);

    if (SchemeCount == 0 || Scheme == -1) {
	QMessageBox::information(this, i18n("Configuration Error"),
				 i18n("There are no schemes defined,\n"
				 "or no scheme is selected."));
        return 0;
    }

    schemes = new QPopupMenu;
    schemes->setCheckable(TRUE);
    connect(schemes, SIGNAL(activated(int)), this, SLOT(schemeChecked(int)));
    schemeID.resize(SchemeCount);

    QString SchemeGroup;
    QString SchemeName;

    for (int i = 0; i < SchemeCount; i++) {
	SchemeGroup.sprintf("Scheme %d", i);
	
	KConfigGroupSaver *group = new KConfigGroupSaver(conf, SchemeGroup);
	SchemeName = conf->readEntry("Description");
	delete (group);

        schemeID[i] = schemes->insertItem(SchemeName);
    }
    schemes->setItemChecked(schemeID[Scheme], TRUE);

    return SchemeCount;
}

void KPacman::quitKPacman()
{
    kapp->quit();
}

void KPacman::newKPacman()
{
    referee->play();
}

void KPacman::pauseKPacman()
{
    referee->pause();
}

void KPacman::togglePaused()
{
    static bool checked = FALSE;
    checked = !checked;
    game->setItemChecked( pauseID, checked );
}

void KPacman::confKeys()
{
    Keys *keys = new Keys();
    if (keys->exec() == QDialog::Accepted)
	referee->initKeys();
    delete keys;
}

void KPacman::schemeChecked(int id)
{
    for (uint i = 0; i < schemeID.size(); i++) {
	schemes->setItemChecked( schemeID[i], schemeID[i] == id);
	if (schemeID[i] == id) {
	    conf->writeEntry("Scheme", i);
	    view->setScheme(conf, i);
	    show();
	}
    }
}

